/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Einen Listenpunkt in der Liste objListe erstellen und verwalten
*/
function Listenpunkt(
    objListe,       // Liste-Instanz, in der der Listenpunkt aufgenommen werden soll
    index           // Index des Textbausteins
) {
    this.liste = objListe;
    this.index = index;
    this.establish();
}

// Zugehörige Liste
Listenpunkt.prototype.liste = null;

// Index des Textbausteins
Listenpunkt.prototype.index = -1;

// HTMLElement, welches Listenpunkt repräsentiert
Listenpunkt.prototype.node = null;

/**
* Darauf reagieren, dass Listenpunkt angeklickt wird
* @return undefined
*/
Listenpunkt.prototype.procClick = function() {
    if (this.liste.procClick(this))
        this.contextmenu();
}

/**
* Kontextmenü anzeigen
* @return undefined
*/
Listenpunkt.prototype.contextmenu = function() {
    var self = this;
    var isEditable = this.liste.getListEditable();
    var arrItems = [];
    if (isEditable == "never") {
        arrItems.push({ label: 'Verwenden', className: 'btn_up', disabled: (this.liste.getEditable() == 'no'), title: 'Das Ausgewählte verwenden', onclick: function() { self.procClick(); } });
    } else {
        isEditable = (isEditable == "yes");
        var isTop = !isEditable || Liste.isTop(this);
        var isBottom = !isEditable || Liste.isBottom(this);
        arrItems = [
            { label: 'Ganz nach oben', className: 'btn_top', title: 'An den ersten Platz der Auflistung verschieben', disabled: isTop, onclick: function() { self.toTop(); } },
            { label: 'Nach oben', className: 'btn_up', title: 'In der Auflistung um einen Platz nach oben verschieben', disabled: isTop, onclick: function() { self.up(); } },
            { label: 'Nach unten', className: 'btn_down', title: 'In der Auflistung um einen Platz nach unten verschieben', disabled: isBottom, onclick: function() { self.down(); } },
            { label: 'Ganz nach unten', className: 'btn_bottom', title: 'An den letzten Platz der Auflistung verschieben', disabled: isBottom, onclick: function() { self.toBottom(); } },
            { label: 'Löschen', className: 'btn_delete', title: 'Aus der Liste Löschen', disabled: !isEditable, onclick: function() { self.requestRemove(); } },
        ];
    }
    arrItems.push({ label: 'Zurück', className: 'btn_prev', title: 'Nichts ändern und dieses Menü schließen' });
    NoBtnBarModalBackground.showBubble(this.node, arrItems);
}

/**
* Listenpunkt anzeigen
* @return undefined
*/
Listenpunkt.prototype.establish = function() {
    var self = this;
    this.node = this.liste.getSpace(this);
    var link = ScoTools.createHtml('a', this.node, { "href": "#" }, this.liste.getText(this.index));
    link.addEventListener('click', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.procClick();
    });
    link.addEventListener('contextmenu', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.contextmenu();
    });
    // Schaltflächen für einfache Editierung?
    if (this.liste.getEditable() == 'yes') {
        this._ensureToollist();
        this.update();
    }
}

/**
* Falls noch nicht vorhanden: Leiste für einfache Editierung und Rückmeldung anlegen
* @return undefined
*/
Listenpunkt.prototype._ensureToollist = function() {
    if (!this._toollist)
        this._toollist = ScoTools.createHtml('span', this.node, { 'class': 'edit'});
}

// Bereich mit Editierwerkzeugen
Listenpunkt.prototype._toollist = null;

// Schaltflächen zum einfachen editieren
Listenpunkt.prototype._tools = [];

/**
* Anklickbarkeit der Schaltflächen ggf. ändern
* @return undefined
*/
Listenpunkt.prototype.update = function() {
    if (!this.node)
        return;
    switch (this.liste.getListEditable()) {
        case 'no':
            this._tools.map(function(btn) { btn.disabled = true; });
        case 'never':
            return;
    }
    if (this.node.firstChild.className)
        this.node.firstChild.className = '';
    if (this._tools.length != 3) {
        this._ensureToollist();
        this._toollist.innerHTML = '';
        var self = this;
        this._tools = [
            new ScoTools.ImageLink('Nach oben', 'share/up.svg', '↑', function() { self.up(); }),
            new ScoTools.ImageLink('Nach unten', 'share/down.svg', '↓', function() { self.down(); }),
            new ScoTools.ImageLink('Löschen', 'share/delete.svg', 'x', function() { self.requestRemove(); })
        ];
        this._tools.map(function(btn) { btn.appendAsChild(self._toollist); });
    }
    this._tools[0].disabled = Liste.isTop(this);
    this._tools[1].disabled = Liste.isBottom(this);
    this._tools[2].disabled = false;
}

/**
* Listenpunkt ganz an den Anfang seiner Liste verschieben
* @return undefined
*/
Listenpunkt.prototype.toTop = function() {
    Liste.toTop(this);
}

/**
* Listenpunkt um eins nach oben bewegen
* @return undefined
*/
Listenpunkt.prototype.up = function() {
    Liste.up(this);
}

/**
* Listenpunkt um eins nach unten bewegen
* @return undefined
*/
Listenpunkt.prototype.down = function() {
    Liste.down(this);
}

/**
* Listenpunkt ganz an das Ende seiner Liste verschieben
* @return undefined
*/
Listenpunkt.prototype.toBottom = function() {
    Liste.toBottom(this);
}

/**
* Darum bitten, dass der Listenpunkt entfernt wird
* @return undefined
*/
Listenpunkt.prototype.requestRemove = function() {
    Liste.requestRemove(this);
}

/**
* Diesen Listenpunkt aus seiner Liste entfernen
* @return undefined
*/
Listenpunkt.prototype.remove = function() {
    Liste.remove(this);
}

/**
* Die y-Koordinate ermitteln, an der der Listenpunkt beginnt
* @return Number        y-Koordinate
*/
Listenpunkt.prototype.getTop = function() {
    return this.node.getBoundingClientRect().top;
}

/**
* Animieren, wie dieser Listenpunkt aus einem anderen hervorgeht
* @return undefined
*/
Listenpunkt.prototype.animate = function(
    top         // Alte y-Koordinate
) {
    try {
        top -= this.getTop();
        this.node.animate([
            { "transform": "translateY(" + top + "px)" },
            { "transform": "translateY(0px)" }
        ], { "duration": 500, "easing": "ease" });
    } catch {}
}

/**
* Die Richtigkeit dieses Listenpunktes kennzeichnen
* @return undefined
*/
Listenpunkt.prototype.markCorrectness = function(
    correctness     // Ist der Listenpunkt richtig (true), falsch (false) oder noch nicht geprüft (undefined)?
) {
    this._tools = [];
    this._ensureToollist();
    this._toollist.innerHTML = '';
    var btn;
    if (correctness === undefined) {
        this.node.firstChild.className = 'uncheckedanswere';
        btn = new ScoTools.ImageLink('Mehr Details anzeigen', 'share/unknown.svg', '?', function() { alert('Ob dieser Textbaustein hier richtig ist, wurde nicht geprüft: Den es gibt einen gravierenderen Fehler.'); });
    } else if (correctness) {
        this.node.firstChild.className = 'correctanswere';
        btn = new ScoTools.ImageLink('Mehr Details anzeigen', 'share/ok.svg', '✔', function() { alert('Dieser Textbaustein ist an dieser Stelle richtig.'); });
    } else {
        this.node.firstChild.className = 'wronganswere';
        btn = new ScoTools.ImageLink('Mehr Details anzeigen', 'share/not-ok.svg', '✘', function() { alert('Dieser Textbaustein an dieser Stelle der erste bzw. gravierendste Fehler.\n\nAchtung: Möglicherweise gibt es weitere Fehler…'); });
    }
    btn.appendAsChild(this._toollist);
}
